<?php

App::uses('BuzzDiaryAccountsAppController', 'BuzzDiaryAccounts.Controller');
App::uses('CustomerAddress', 'BuzzCustomers.Model');

class CampBookingsController extends BuzzDiaryAccountsAppController {

/**
 * Components
 *
 * @var array
 */
	public $components = array(
		'BuzzPurchase.Purchase',
		'Transactions.Transactions'
	);

/**
 * Book and pay for a camp.
 *
 * @param int $campId ID of camp being booked
 * @return void
 */
	public function book($campId) {
		$CampBooking = $this->{$this->modelClass};

		$data = $CampBooking->Camp->getBooking($campId);
		if (empty($data)) {
			throw new NotFoundException();
		} elseif ($data['Camp']['remaining_availability'] === 0) {
			$this->Session->setFlash(__d('buzz_camps', 'Sorry, this camp is now fully booked'));
			return $this->redirect(['controller' => 'camps', 'action' => 'index']);
		}

		$this->set(compact('data'));

		if (!empty($this->request->data)) {

			if ($CampBooking->createBooking($campId, $this->request->data) === true) {
				// Take payment.
				$this->_pay();
			} else {
				$this->Session->setFlash(
					__d('buzz_camps', 'Please correct the errors below'),
					'flash_fail'
				);
			}

		} else {

			// Set the address field defaults (these are configured by the
			// CustomerAddress plugin to make it easier to override for each site).
			$this->request->data['AccountCredit']['CustomerAddress']['country_id'] = CustomerAddress::getDefaultCountry();
			$this->request->data['AccountCredit']['CustomerAddress']['us_state_id'] = CustomerAddress::getDefaultUsState();

		}

		$this->_populateLookups();
		$this->view = 'BuzzCamps./CampBookings/book';

		return;
	}

/**
 * Processes the payment and triggers the completion of the payment on success
 *
 * @return void
 */
	protected function _pay() {
		$CampBooking = $this->{$this->modelClass};

		// Get the latest basket.
		$id = $CampBooking->getBasketId();
		$basket = $CampBooking->find(
			'first',
			array(
				'contain' => array(
					'AccountCredit' => array(
						'CustomerAddress' => array(
							'Country'
						)
					)
				),
				'conditions' => array(
					'CampBooking.id' => $id
				)
			)
		);

		if ($basket['CampBooking']['total_paid'] > 0) {

			if (Configure::read('BuzzPurchase.onsite') === true) {
				$payment = $this->request->data['Payment'];

				// Split name into first and second names for payment gateways.
				$fullName = trim($payment['card_name']);
				$splitName = explode(' ', $fullName);
				$payment['last_name'] = array_pop($splitName);
				$payment['first_name'] = count($splitName) ? implode(' ', $splitName) : '';
			} else {
				$payment = [];
			}

			$transaction = $this->Transactions->takePayment(
				// Return URLs
				array(
					'return' => Router::url(['action' => 'payment_callback'], true),
					'cancel' => Router::url(['action' => 'book', $basket['CampBooking']['camp_id']], true)
				),
				// Calling record
				array(
					'model' => 'BuzzCamps.CampBooking',
					'model_id' => $basket['CampBooking']['id']
				),
				// Amount to be paid
				$basket['CampBooking']['total_paid'],
				// Items
				array(),
				// Extras
				array(
					'language' => Configure::read('Config.language'),
					'card' => $payment,
					'address' => $basket['AccountCredit']['CustomerAddress'],
					'user' => array('email' => $basket['AccountCredit']['CustomerAddress']['email']),
					'description' => __d(
						'buzz_camps',
						'%s Camp Booking %d',
						[Configure::read('SiteSetting.site_title'), $basket['CampBooking']['id']]
					)
				),
				0,
				Configure::read('BuzzPurchase.payment_gateway')
			);

			if (!empty($transaction['result'])) {
				// Payment taken successfully, complete purchase.
				return $this->_completeBooking();
			} else {
				$this->Session->setFlash(
					__d('buzz_camps', 'There was a problem processing your payment, please check your details and try again'),
					'flash_fail'
				);
			}

		} else {
			// Nothing to pay, complete purchase.
			return $this->_completeBooking();
		}


		return;
	}

/**
 * Generic payment gateway callback.
 *
 * @return void
 */
	public function payment_callback() {
		$result = $this->Transactions->checkPayment();

		if ($result === true) {
			return $this->_completeBooking();
		} else {
			$this->Session->setFlash(
				__d('buzz_camps', 'There was a problem processing your payment, please try again'),
				'flash_fail'
			);
			return $this->redirect([
				'action' => 'index'
			]);
		}
	}

/**
 * Complete the credit transaction (after payment received)
 *
 * @return void Redirects to confirmation page
 */
	protected function _completeBooking() {
		$CampBooking = $this->{$this->modelClass};

		$basketId = $CampBooking->getBasketId();

		if (empty($basketId)) {
			throw new ForbiddenException();
		}

		$this->Session->write('CampBooking.hash', $CampBooking->hashBasketId());

		$CampBooking->completeBooking(true);

		return $this->redirect([
			'action' => 'confirmation',
			$basketId
		]);
	}

/**
 * Credit confirmation page.
 *
 * @return void
 */
	public function confirmation($id) {
		$CampBooking = $this->{$this->modelClass};

		if ($CampBooking->hashBasketId($id) === $this->Session->read('CampBooking.hash')) {

			$this->Session->delete('CampBooking.hash');

			$booking = $CampBooking->readForView($id);

			if (Configure::check('BuzzCamps.camp_booking_confirmation_page_id') === true) {
				$this->assignPage(Configure::read('BuzzCamps.camp_booking_confirmation_page_id'));
			}

			$this->set(compact('booking'));
			$this->view = 'BuzzCamps.confirmation';

		} else {

			throw new ForbiddenException();

		}

		return;
	}

/**
 * Populate lookups.
 *
 * @return void
 */
	protected function _populateLookups() {
		$CampBooking = $this->{$this->modelClass};

		$countries = $CampBooking->CustomerAddress->Country->translatedList();
		$usStates = $CampBooking->CustomerAddress->UsState->find('list');

		$this->set('cardTypes', $this->Purchase->getCardTypes());
		$this->set('months', $this->Purchase->getMonths());
		$this->set('years', $this->Purchase->getYears());
		$this->set('pastYears', $this->Purchase->getPastYears());

		$this->set(compact('countries', 'usStates'));
		return;
	}

}
